from ScoutSuite.core.console import print_exception
from ScoutSuite.providers.aws.facade.basefacade import AWSBaseFacade
from ScoutSuite.providers.aws.facade.utils import AWSFacadeUtils
from ScoutSuite.providers.utils import map_concurrently, run_concurrently


class AcmFacade(AWSBaseFacade):
    async def get_certificates(self, region):
        try:
            cert_list = await AWSFacadeUtils.get_all_pages('acm', region, self.session, 'list_certificates', 'CertificateSummaryList')
            cert_arns = [cert['CertificateArn'] for cert in cert_list]
        except Exception as e:
            print_exception('Failed to get acm certificates: {}'.format(e))
            return []
        else:
            return await map_concurrently(self._get_certificate, cert_arns, region=region)

    async def _get_certificate(self, cert_arn: str, region: str):
        client = AWSFacadeUtils.get_client('acm', self.session, region)
        try:
            return await run_concurrently(lambda: client.describe_certificate(CertificateArn=cert_arn)['Certificate'])
        except Exception as e:
            print_exception('Failed to describe acm certificate: {}'.format(e))
            raise




from ScoutSuite.providers.azure.resources.base import AzureResources


class Groups(AzureResources):
    async def fetch_all(self):
        for raw_group in await self.facade.graphrbac.get_groups():
            id, group = self._parse_group(raw_group)
            self[id] = group

    def _parse_group(self, raw_group):
        group_dict = {}
        group_dict['id'] = raw_group.object_id
        group_dict['name'] = raw_group.display_name
        group_dict['additional_properties'] = raw_group.additional_properties
        group_dict['deletion_timestamp'] = raw_group.deletion_timestamp
        group_dict['object_type'] = raw_group.object_type
        group_dict['mail_enabled'] = raw_group.mail_enabled
        group_dict['mail_nickname'] = raw_group.mail_nickname
        group_dict['security_enabled'] = raw_group.security_enabled
        group_dict['mail'] = raw_group.mail
        group_dict['users'] = []  # this will be filled in `finalize()`
        group_dict['roles'] = []  # this will be filled in `finalize()`
        return group_dict['id'], group_dict




from ScoutSuite.providers.azure.resources.base import AzureResources


class Roles(AzureResources):
    async def fetch_all(self):
        for raw_role in await self.facade.graphrbac.get_roles():
            id, role = self._parse_role(raw_role)
            self[id] = role

    def _parse_role(self, raw_role):
        role_dict = {}
        role_dict['id'] = raw_role.name
        role_dict['name'] = raw_role.role_name
        role_dict['type'] = raw_role.type
        role_dict['description'] = raw_role.description
        role_dict['role_type'] = raw_role.role_type
        role_dict['permissions'] = raw_role.permissions
        role_dict['assignable_scopes'] = raw_role.assignable_scopes
        role_dict['additional_properties'] = raw_role.additional_properties
        role_dict['assignments_count'] = 0
        role_dict['assignments'] = {'users': [],
                                    'groups': [],
                                    'service_principals': []}  # this will be filled in `finalize()`
        return role_dict['id'], role_dict



from ScoutSuite.providers.azure.resources.base import AzureCompositeResources

from .users import Users
from .groups import Groups
from .serviceprincipals import ServicePrincipals
from .applications import Applications
from .roles import Roles


class GraphRBAC(AzureCompositeResources):
    _children = [
        (Users, 'users'),
        (Groups, 'groups'),
        (ServicePrincipals, 'service_principals'),
        (Applications, 'applications'),
        (Roles, 'roles')
    ]

    async def fetch_all(self):
        await self._fetch_children(resource_parent=self)

    async def finalize(self):

        # Add group members
        for group in self['groups']:
            for user in self['users']:
                if group in self['users'][user]['groups']:
                    self['groups'][group]['users'].append(user)

        # Add role assignments
        assignments = await self.facade.graphrbac.get_role_assignments()
        for assignment in assignments:
            role_id = assignment.role_definition_id.split('/')[-1]
            for group in self['groups']:
                if group == assignment.principal_id:
                    self['groups'][group]['roles'].append(role_id)
                    self['roles'][role_id]['assignments']['groups'].append(group)
                    self['roles'][role_id]['assignments_count'] += 1
            for user in self['users']:
                if user == assignment.principal_id:
                    self['users'][user]['roles'].append(role_id)
                    self['roles'][role_id]['assignments']['users'].append(user)
                    self['roles'][role_id]['assignments_count'] += 1
            for service_principal in self['service_principals']:
                if service_principal == assignment.principal_id:
                    self['service_principals'][service_principal]['roles'].append(role_id)
                    self['roles'][role_id]['assignments']['service_principals'].append(service_principal)
                    self['roles'][role_id]['assignments_count'] += 1



from ScoutSuite.providers.azure.resources.base import AzureResources


class InformationProtectionPolicies(AzureResources):
    async def fetch_all(self):
        for raw_information_policies in await self.facade.securitycenter.get_information_protection_policies():
            id, information_protection_policies = self._parse_information_protection_policies(
                raw_information_policies)
            self[id] = information_protection_policies

    def _parse_information_protection_policies(self, auto_provisioning_settings):
        information_protection_policies_dict = {}
        return information_protection_policies_dict['id'], information_protection_policies_dict



from ScoutSuite.providers.aws.facade.base import AWSFacade
from ScoutSuite.providers.aws.resources.base import AWSResources

from ScoutSuite.providers.utils import get_non_provider_id


class Certificates(AWSResources):
    def __init__(self, facade: AWSFacade, region: str):
        super(Certificates, self).__init__(facade)
        self.region = region

    async def fetch_all(self):
        raw_certificates = await self.facade.acm.get_certificates(self.region)
        for raw_certificate in raw_certificates:
            name, resource = self._parse_certificate(raw_certificate)
            self[name] = resource

    def _parse_certificate(self, raw_certificate):
        raw_certificate['name'] = raw_certificate.get('DomainName')
        raw_certificate['id'] = get_non_provider_id(raw_certificate['name'])

        return raw_certificate['id'], raw_certificate



_strategies = {
    'aws': 'AWSAuthenticationStrategy',
    'gcp': 'GCPAuthenticationStrategy',
    'azure': 'AzureAuthenticationStrategy',
    'aliyun': 'AliyunAuthenticationStrategy',
    'oci': 'OracleAuthenticationStrategy'
}


def import_authentication_strategy(provider):
    strategy_class = _strategies[provider]
    module = __import__('ScoutSuite.providers.{}.authentication_strategy'.format(provider), fromlist=[strategy_class])
    authentication_strategy = getattr(module, strategy_class)
    return authentication_strategy


def get_authentication_strategy(provider: str):
    """
        Returns an authentication strategy implementation for a provider.
        :param provider: The authentication strategy 
    """
    authentication_strategy = import_authentication_strategy(provider)
    return authentication_strategy()



from ScoutSuite.providers.aws.facade.base import AWSFacade
from ScoutSuite.providers.aws.resources.regions import Regions

from .certificates import Certificates


class Certificates(Regions):
    _children = [
        (Certificates, 'certificates')
    ]

    def __init__(self, facade: AWSFacade):
        super(Certificates, self).__init__('acm', facade)



from ScoutSuite.providers.azure.resources.base import AzureResources
from ScoutSuite.providers.utils import get_non_provider_id


class ApplicationSecurityGroups(AzureResources):
    async def fetch_all(self):
        for raw_group in await self.facade.network.get_application_security_groups():
            id, application_security_group = self._parse_application_security_group(raw_group)
            self[id] = application_security_group

    def _parse_application_security_group(self, raw_application_security_group):
        application_security_group_dict = {}
        application_security_group_dict['id'] = get_non_provider_id(raw_application_security_group.id)
        application_security_group_dict['name'] = raw_application_security_group.name
        application_security_group_dict['type'] = raw_application_security_group.type
        application_security_group_dict['location'] = raw_application_security_group.location
        application_security_group_dict['tags'] = raw_application_security_group.tags
        application_security_group_dict['resource_guid'] = raw_application_security_group.resource_guid
        application_security_group_dict['provisioning_state'] = raw_application_security_group.provisioning_state
        application_security_group_dict['etag'] = raw_application_security_group.etag
        application_security_group_dict['network_interfaces'] = []  # this is filled in the base class
        return application_security_group_dict['id'], application_security_group_dict




from ScoutSuite.providers.azure.resources.base import AzureResources


class Applications(AzureResources):
    async def fetch_all(self):
        for raw_application in await self.facade.graphrbac.get_applications():
            id, application = await self._parse_application(raw_application)
            self[id] = application

    async def _parse_application(self, raw_application):
        application_dict = {}
        application_dict['id'] = raw_application.object_id
        application_dict['app_id'] = raw_application.app_id
        application_dict['name'] = raw_application.display_name
        application_dict['additional_properties'] = raw_application.additional_properties
        application_dict['deletion_timestamp'] = raw_application.deletion_timestamp
        application_dict['object_type'] = raw_application.object_type
        application_dict['allow_guests_sign_in'] = raw_application.allow_guests_sign_in
        application_dict['allow_passthrough_users'] = raw_application.allow_passthrough_users
        application_dict['app_logo_url'] = raw_application.app_logo_url
        application_dict['app_roles'] = raw_application.app_roles
        application_dict['app_permissions'] = raw_application.app_permissions
        application_dict['available_to_other_tenants'] = raw_application.available_to_other_tenants
        application_dict['error_url'] = raw_application.error_url
        application_dict['group_membership_claims'] = raw_application.group_membership_claims
        application_dict['homepage'] = raw_application.homepage
        application_dict['identifier_uris'] = raw_application.identifier_uris
        application_dict['informational_urls'] = raw_application.informational_urls
        application_dict['is_device_only_auth_supported'] = raw_application.is_device_only_auth_supported
        application_dict['key_credentials'] = raw_application.key_credentials
        application_dict['known_client_applications'] = raw_application.known_client_applications
        application_dict['logout_url'] = raw_application.logout_url
        application_dict['oauth2_allow_implicit_flow'] = raw_application.oauth2_allow_implicit_flow
        application_dict['oauth2_allow_url_path_matching'] = raw_application.oauth2_allow_url_path_matching
        application_dict['oauth2_permissions'] = raw_application.oauth2_permissions
        application_dict['oauth2_require_post_response'] = raw_application.oauth2_require_post_response
        application_dict['org_restrictions'] = raw_application.org_restrictions
        application_dict['optional_claims'] = raw_application.optional_claims
        application_dict['password_credentials'] = raw_application.password_credentials
        application_dict['pre_authorized_applications'] = raw_application.pre_authorized_applications
        application_dict['public_client'] = raw_application.public_client
        application_dict['publisher_domain'] = raw_application.publisher_domain
        application_dict['reply_urls'] = raw_application.reply_urls
        application_dict['required_resource_access'] = raw_application.required_resource_access
        application_dict['saml_metadata_url'] = raw_application.saml_metadata_url
        application_dict['sign_in_audience'] = raw_application.sign_in_audience
        application_dict['www_homepage'] = raw_application.www_homepage
        return application_dict['id'], application_dict



import oss2
from aliyunsdkcore.client import AcsClient

from ScoutSuite.core.console import print_exception


def get_client(credentials, region=None):
    try:
        client = AcsClient(credential=credentials.credentials, region_id=region if region else 'cn-hangzhou')
        return client

    except Exception as e:
        print_exception(e)
        return None


def get_oss_client(credentials, region=None):
    try:
        auth = oss2.Auth(credentials.credentials.access_key_id, credentials.credentials.access_key_secret)
        client = oss2.Service(auth,
                              endpoint='oss-{}.aliyuncs.com'.format(region) if region
                              else 'oss-cn-hangzhou.aliyuncs.com')
        return client

    except Exception as e:
        print_exception(e)
        return None



from ScoutSuite.providers.aws.facade.base import AWSFacade
from ScoutSuite.providers.aws.resources.base import AWSResources


class Functions(AWSResources):
    def __init__(self, facade: AWSFacade, region: str):
        super(Functions, self).__init__(facade)
        self.region = region

    async def fetch_all(self):
        raw_functions = await self.facade.awslambda.get_functions(self.region)
        for raw_function in raw_functions:
            name, resource = self._parse_function(raw_function)
            self[name] = resource

    def _parse_function(self, raw_function):

        function_dict = {}
        function_dict['name'] = raw_function.get('FunctionName')
        function_dict['arn'] = raw_function.get('FunctionArn')
        function_dict['runtime'] = raw_function.get('Runtime')
        function_dict['role'] = raw_function.get('Role')
        function_dict['handler'] = raw_function.get('Handler')
        function_dict['code_size'] = raw_function.get('CodeSize')
        function_dict['description'] = raw_function.get('Description')
        function_dict['timeout'] = raw_function.get('Timeout')
        function_dict['memory_size'] = raw_function.get('MemorySize')
        function_dict['last_modified'] = raw_function.get('LastModified')
        function_dict['code_sha256'] = raw_function.get('CodeSha256')
        function_dict['version'] = raw_function.get('Version')
        function_dict['tracing_config'] = raw_function.get('TracingConfig')
        function_dict['revision_id'] = raw_function.get('RevisionId')
        return function_dict['name'], function_dict

